/*
 * Created on 23.09.2008
 * 
 */
package com.eaio.twitterbackup;

import static com.eaio.util.Resource.close;

import java.awt.Desktop;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.swing.JOptionPane;

import oauth.signpost.OAuth;
import oauth.signpost.OAuthConsumer;
import oauth.signpost.OAuthProvider;
import oauth.signpost.basic.DefaultOAuthConsumer;
import oauth.signpost.basic.DefaultOAuthProvider;
import oauth.signpost.exception.OAuthException;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.eaio.util.text.HumanTime;
import com.thoughtworks.xstream.XStreamException;

/**
 * Twitter backup controller should be run in a separate thread to fetch twitter updates and write them to an output
 * file when done.
 * 
 * @author <a href="mailto:jb&#64;eaio.com">Johann Burkard</a>
 * @version $Id: TwitterBackupController.java 6465 2014-07-04 22:00:32Z johann $
 */
public class TwitterBackupController implements Runnable {

    private static final Logger LOG = Logger.getLogger(TwitterBackupController.class);

    private TwitterBackup backup;

    private String username, token, tokenSecret, password, proxy, file;

    private char port;

    private long timeout = 10500;

    private final OAuthConsumer consumer = new DefaultOAuthConsumer("neibRfmUvzUnqjKL3rDkpA",
            "Hq02kO6znwoHiJOwnQAVApBgZH8m4JuHRIbB3EILU");

    private final OAuthProvider provider = new DefaultOAuthProvider("https://api.twitter.com/oauth/request_token",
            "https://api.twitter.com/oauth/access_token", "https://api.twitter.com/oauth/authorize");

    /**
     * Constructor for TwitterBackupController.
     * 
     */
    public TwitterBackupController() {
        this(new TwitterBackup());
    }

    /**
     * Constructor for TwitterBackupController.
     * 
     * @param backup the backup to use
     */
    public TwitterBackupController(TwitterBackup backup) {
        super();
        this.backup = backup;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = StringUtils.trimToNull(username);
        this.token = this.tokenSecret = null;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = StringUtils.trimToNull(password);
        this.token = this.tokenSecret = null;
    }

    /**
     * @see java.lang.Runnable#run()
     */
    public void run() {

        if (getToken() == null || getTokenSecret() == null) {
            try {
                String authURL = provider.retrieveRequestToken(consumer, OAuth.OUT_OF_BAND);
                String infoMsg = "opening " + authURL + " in your browser",
                    errMsg = "looks like TwitterBackup cannot start your browser, please go to " + authURL;
                try {
                    System.out.println(infoMsg);
                    LOG.info(infoMsg);
                    Desktop.getDesktop().browse(new URI(authURL));
                }
                catch (IOException ex) {
                    System.err.println(errMsg);
                    LOG.warn(errMsg);
                }
                catch (URISyntaxException ex) {
                    System.err.println(errMsg);
                    LOG.warn(errMsg);
                }
                catch (UnsupportedOperationException ex) {
                    System.err.println(errMsg);
                    LOG.warn(errMsg);
                }
                String pin = StringUtils.trimToEmpty(JOptionPane.showInputDialog("Please enter your PIN"));
                provider.retrieveAccessToken(consumer, pin);
                
                token = consumer.getToken();
                tokenSecret = consumer.getTokenSecret();
            }
            catch (OAuthException ex) {
                LOG.error(ex.getLocalizedMessage() + ", aborting");
                return;
            }
        }
        else {
            consumer.setTokenWithSecret(token, tokenSecret);
        }

        long then = System.currentTimeMillis();

        readExistingBackup();

        boolean changed = false;
        int page = 1;
        int read = 0;
        BufferedWriter writer = null;
        URL u = null;

        do {
            try {
                u = new URL("https://api.twitter.com/1.1/statuses/user_timeline.json?screen_name=" + username + "&page=" + page + "&count=200");
                if (LOG.isInfoEnabled()) {
                    LOG.info("loading " + u);
                }
                read = -1;
                read = backup.readFromJSON(openURL(u));
                if (read > 0) {
                    if (LOG.isInfoEnabled()) {
                        LOG.info(read + " new tweets downloaded, " + backup.size() + " total");
                    }
                    changed = true;
                }
                else {
                    LOG.info("no new tweets found");
                }
            }
            catch (IllegalStateException ex) {
                LOG.warn(ex.getLocalizedMessage() + ", will try again");
            }
            catch (MalformedURLException ex) {
                LOG.error(ex.getLocalizedMessage() + ", aborting");
                break;
            }
            catch (IOException ex) {
                LOG.warn(ex.getLocalizedMessage() + ", will try again");
            }
            catch (OAuthException ex) {
                LOG.error(ex.getLocalizedMessage() + ", aborting");
                break;
            }
            catch (XStreamException ex) {
                LOG.error("Twitter has changed their API again. Please email johann@johannburkard.de, thanks.");
                ex.printStackTrace();
                break;
            }
            if (read == -1 || read > 0) {
                if (LOG.isInfoEnabled()) {
                    LOG.info("waiting " + HumanTime.approximately(timeout));
                }
                try {
                    Thread.sleep(timeout);
                }
                catch (InterruptedException ex) {
                    break;
                }
            }
            if (read > 0) {
                ++page;
            }
        }
        while (read != 0);

        try {
            if (changed) {
                if (LOG.isInfoEnabled()) {
                    LOG.info("saving backup to " + getFile());
                }
                writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(getFile()), "UTF-8"));
                writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
                backup.write(writer);
            }
        }
        catch (IOException ex) {
            LOG.error("couldn't write to " + getFile() + ": " + ex.getLocalizedMessage());
        }
        finally {
            close(writer);
        }

        if (LOG.isInfoEnabled()) {
            LOG.info("backup finished after " + HumanTime.approximately(System.currentTimeMillis() - then));
        }
    }

    private void readExistingBackup() {
        if (getFile() != null) {
            File existing = new File(getFile());
            if (existing.isFile() && existing.canRead()) {
                Reader reader = null;
                try {
                    reader = new InputStreamReader(new FileInputStream(existing), "UTF-8");
                    backup.clear();
                    int read = backup.readFromXML(reader);
                    LOG.info("read " + read + " tweets from " + getFile());
                }
                catch (IOException ex) {
                    LOG.warn("could not read " + getFile() + ": " + ex.getLocalizedMessage());
                }
                finally {
                    close(reader);
                }
            }
        }
    }

    public void save() throws BackingStoreException {
        Preferences prefs = Preferences.userNodeForPackage(getClass());
        prefs.put("username", StringUtils.defaultString(username));
        prefs.put("password", StringUtils.defaultString(password));
        prefs.put("proxy", StringUtils.defaultString(proxy));
        prefs.putInt("port", port);
        prefs.put("file", StringUtils.defaultString(file));
        prefs.remove("maxPages"); // From TwitterBackup 2, unused now.
        prefs.remove("timeout");
        if (token == null) {
            prefs.remove("token");
        }
        else {
            prefs.put("token", token);
        }
        if (tokenSecret == null) {
            prefs.remove("tokenSecret");
        }
        else {
            prefs.put("tokenSecret", tokenSecret);
        }
        prefs.flush();
    }

    public void restore() {
        Preferences prefs = Preferences.userNodeForPackage(getClass());
        username = prefs.get("username", null);
        password = prefs.get("password", null);
        proxy = StringUtils.trimToNull(prefs.get("proxy", null));
        port = (char) prefs.getInt("port", 0);
        file = prefs.get("file", null);
        token = prefs.get("token", null);
        tokenSecret = prefs.get("tokenSecret", null);
    }

    public String getFile() {
        return file;
    }

    public void setFile(String file) {
        this.file = StringUtils.trimToNull(file);
    }

    public char getPort() {
        return port;
    }

    public void setPort(char port) {
        this.port = port;
    }

    public String getProxy() {
        return proxy;
    }

    public void setProxy(String proxy) {
        this.proxy = StringUtils.trimToNull(proxy);
    }

    public long getTimeout() {
        return timeout;
    }

    public void setTimeout(long timeout) {
        this.timeout = timeout;
    }

    public String getToken() {
        return token;
    }

    public void setToken(String accessToken) {
        this.token = accessToken;
    }

    public String getTokenSecret() {
        return tokenSecret;
    }

    public void setTokenSecret(String tokenSecret) {
        this.tokenSecret = tokenSecret;
    }

    private Reader openURL(URL u) throws IOException, IllegalStateException, OAuthException {
        if (proxy != null) {
            System.setProperty("http.proxySet", Boolean.TRUE.toString());
            System.setProperty("http.proxyHost", proxy);
            System.setProperty("http.proxyPort", Integer.toString(port));
        }
        HttpURLConnection conn = (HttpURLConnection) u.openConnection();
        conn.setRequestProperty("User-Agent", TwitterBackup.VERSION + " by Johann Burkard; +http://is.gd/4ete");
        consumer.sign(conn);
        int response = conn.getResponseCode();
        if (response != 200) {
            throw new IllegalStateException(conn.getResponseCode() + ": " + conn.getResponseMessage());
        }
        return new InputStreamReader(conn.getInputStream(), "UTF-8");
    }

}
